﻿using Infragistics.Win.Layout;
using Infragistics.Win.Misc;
using Infragistics.Win.UltraWinEditors;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace WindowsFormsApp37
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            this.ultraPanel1.AutoScroll = true;

            int questionCount = 13;
            int halfwayMark = questionCount / 2;
            Point location = new Point(8, 8);
            for (int i = 0; i < questionCount; i++)
            {
                this.AddGroupBox(i);
            }

            this.PositionQuestions();
        }

        private void AddGroupBox(int index)
        {
            var gb = new GroupBox();
            gb.Text = string.Format("Question #{0}", index);
            gb.AutoSize = true;

            var numberOfOptions = DummyDataCreator.RandomInt(2, 4);
            Point p = new Point(gb.ClientRectangle.Left + 8, gb.ClientRectangle.Top + 16);
            for (int i = 0; i < numberOfOptions; i++)
            {
                var option = new UltraRadioButton();
                option.Text = string.Format("Option #{0}", i.ToString());
                option.AutoSize = true;
                option.Location = p;
                p.Y = p.Y + option.Height + 8;
                gb.Controls.Add(option);
            }

            this.ultraPanel1.ClientArea.Controls.Add(gb);
        }

        private void ultraPanel1_Resize(object sender, EventArgs e)
        {
            this.PositionQuestions();
        }

        private void PositionQuestions()
        {
            int padding = 8;
            var clientArea = this.ultraPanel1.ClientArea;
            var availableSize = clientArea.ClientSize;

            // Leave space for the scroll bar. Ideally, we would only do this when a scollbar is 
            // needed, but that qwould require a two-pass process, so it's a lot easier to 
            // just reserve the space. 
            availableSize.Width -= SystemInformation.VerticalScrollBarWidth;

            // Also leave a little padding
            availableSize.Width -= padding;

            // Figure out the maximum width of a question, so we can determine the number of columns. 
            int maxWidth = 0;
            foreach (Control control in clientArea.Controls)
            {
                // The padding here is to account for some space between each panel.
                maxWidth = Math.Max(maxWidth, control.Width + padding);
            }

            int questionCount = clientArea.Controls.Count;
            int numberOfColumns = availableSize.Width / maxWidth;
            int itemsPerColumn = questionCount / numberOfColumns;
            int remainder = questionCount % numberOfColumns;

            int[] questionsPerColumn = new int[numberOfColumns];
            for (int column = 0; column < numberOfColumns; column++)
            {
                questionsPerColumn[column] = itemsPerColumn;
                if (remainder > 0)
                {
                    questionsPerColumn[column]++;
                    remainder--;
                }
            }

            Point location = new Point(padding, padding);
            int index = 0;
            for (int column = 0; column < numberOfColumns; column++)
            {
                location.Y = padding;                

                int numberOfQuestionsInThisColumn = questionsPerColumn[column];
                for (int row = 0; row < numberOfQuestionsInThisColumn; row++)
                {
                    var control = clientArea.Controls[index];
                    index++;
                    control.Location = location;
                    location.Y += control.Height + padding;
                }

                location.X += maxWidth;
            }
        }
    }
}
